//----------------------------------------------------------------------
// wavefamemwrite
// version 1.0
// F. Abignoli
// 4/5/2013
//----------------------------------------------------------------------
// Initialisation de l'EEPROM pour le synthetiseur a table onde
// Fabrique un nom pour chaque preset
// Initialise les donnees de base
// Programme prevu pour 24LC256
//----------------------------------------------------------------------


#include <Wire.h> //I2C library
#include <LiquidCrystal_I2C.h>
#if defined(ARDUINO) && ARDUINO >= 100
#define printByte(args)  write(args);
#else
#define printByte(args)  print(args,BYTE);
#endif

LiquidCrystal_I2C lcd(0x27,16,2);       // sLCD adress 0x27, 16 car, 2 lignes



#define MAXPROG 128
#define PRESETSIZE  64

#define pATTAQUE 0		// Phase attaque en cours
#define pDECAY   1		// Phase de decay en cours
#define pSUSTAIN 2		// Phase de sustain en cours
#define pRELEASE 3		// Phase de release en cours
char nompreset[17];   					// Nom du preset
uint16_t val_ADSR[] = {0,170,300,210};	
byte VCA_OSC1=100;			// Coefficient multiplicateur pour l'onde 1
byte VCA_OSC2=100;			// Coefficient multiplicateur pour l'onde 1
byte lfo1=0;				// frequence du LFO 1
byte lfo2=0;				// frequence du LFO 2
byte depth_lfo1=127;				// profondeur lfo1
byte depth_lfo2=127;				// profondeur lfo2
byte dest_lfo1=0;					// destination lfo1
byte dest_lfo2=1;					// destination lfo2
byte notable[] = {1,12};					// Numero table onde a charger ou a calculer (sinus par defaut)
byte pwm[] = {126,132};					// valeur de PWM (128= square, valeur min 10, valeur max 245)
byte unison=0;							// Mode unison 
byte depth_unison=1;					// profondeur unison
byte semi[] = {12,12};	


void eewrite_byte( int deviceaddress, unsigned int eeaddress, byte data ) 
{
int rdata = data;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.write(rdata);
  Wire.endTransmission();
  delay(5);
}

byte eeread_byte( int deviceaddress, unsigned int eeaddress ) {
  byte rdata = 0xFF;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.endTransmission();
  Wire.requestFrom(deviceaddress,1);
  if (Wire.available()) rdata = Wire.read();
    return rdata;
}

void setup() 
{
  //Wire.begin(); // initialise the connection
  lcd.init();             // init du lcd 
  lcd.backlight();        // Retroeclairage
  lcd.clear();            // Effacement de lecran
  lcd.setCursor(0,0);
}

void init_preset(byte mypreset)
{
int i;
int myaddress=0;
char bufnum[4];
int n;
    // Fabrication dun nom bidon au format
    // "PRESET xxx-Bx   "
    strcpy(nompreset,"PRESET ");
	
    if (mypreset+1<10) {
        strcat(nompreset,"00");
    }
    else {
      if (mypreset+1<100) {
        strcat(nompreset,"0");
      }
    }
    itoa(mypreset+1,bufnum,10);
    strcat(nompreset,bufnum);
    strcat(nompreset,"      ");
	
    // Initialisation de l'adresse du preset
    myaddress=(mypreset*PRESETSIZE);
	// Ecrit nom preset
    for (i=0;i<16;i++) {
      eewrite_byte( 0x50, myaddress++, nompreset[i]);
    }
	
	// numero de tables
    eewrite_byte( 0x50, myaddress++, notable[0]);						// Numero table onde 1
    eewrite_byte( 0x50, myaddress++, notable[1]);						// Numero table onde 2
	
    // Enveloppe
    eewrite_byte( 0x50, myaddress++, highByte(val_ADSR[pATTAQUE]));		// MSB attaque
    eewrite_byte( 0x50, myaddress++, lowByte(val_ADSR[pATTAQUE]));		// LSB attaque
    eewrite_byte( 0x50, myaddress++, highByte(val_ADSR[pDECAY]));		// MSB decay
    eewrite_byte( 0x50, myaddress++, lowByte(val_ADSR[pDECAY]));		// LSB decay
    eewrite_byte( 0x50, myaddress++, highByte(val_ADSR[pSUSTAIN]));		// MSB sustain
    eewrite_byte( 0x50, myaddress++, lowByte(val_ADSR[pSUSTAIN]));		// LSB sustain
    eewrite_byte( 0x50, myaddress++, highByte(val_ADSR[pRELEASE]));		// MSB release
    eewrite_byte( 0x50, myaddress++, lowByte(val_ADSR[pRELEASE]));		// LSB release
	
	// Unison
    eewrite_byte( 0x50, myaddress++, unison);			// Mode unison (on ou off)
    eewrite_byte( 0x50, myaddress++, depth_unison);		// Profondeur unison de 1 a 5
	
	// PWM
    eewrite_byte( 0x50, myaddress++, pwm[0]);			// Pwm oscillateur 1
    eewrite_byte( 0x50, myaddress++, pwm[1]);			// Pwm oscillateur 2
	
	// Amps
    eewrite_byte( 0x50, myaddress++, VCA_OSC1);			// Ampli osc1 (25, 50 ou 100%)
    eewrite_byte( 0x50, myaddress++, VCA_OSC2);			// Ampli osc2 (25, 50 ou 100%)
	
	// Semi-tone
    eewrite_byte( 0x50, myaddress++, semi[0]);			// semitone osc1 en note midi (de 0 a 24)
    eewrite_byte( 0x50, myaddress++, semi[1]);			// semitone osc2 en note midi (de 0 a 24)
	
	// Lfos
    eewrite_byte( 0x50, myaddress++, lfo1);				// Vitesse lfo1
    eewrite_byte( 0x50, myaddress++, lfo2);				// Vitesse lfo2
    eewrite_byte( 0x50, myaddress++, depth_lfo1);		// profondeur lfo1
    eewrite_byte( 0x50, myaddress++, depth_lfo2);		// profondeur lfo2
    eewrite_byte( 0x50, myaddress++, dest_lfo1);		// destination lfo1
    eewrite_byte( 0x50, myaddress++, dest_lfo2);		// destination lfo2
}

void loop() 
{
int addr=0; // adresse de debut
int bank;   // Numero de bank
int i,j,k;  // variables de travail
	lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("Init. EEPROM...");
    lcd.setCursor(0,1);
    lcd.print("Preset..:");
    for (i=0;i<128;i++) {
        lcd.setCursor(10,1);
        lcd.print(i,DEC);
		lcd.print("  ");
        init_preset(i);
    }
    lcd.clear();
    lcd.print("Fin de l'ecriture");
    while(1) {
    }
}